/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "merc.h"

extern time_t environment_update_time;
extern int environment_interval;



/*
 * Local functions.
 */
int	hit_gain	args( ( CHAR_DATA *ch ) );
int	mana_gain	args( ( CHAR_DATA *ch ) );
int	move_gain	args( ( CHAR_DATA *ch ) );
void	mobile_update	args( ( void ) );
void	weather_update	args( ( void ) );
void	char_update	args( ( void ) );
void	pc_update	args( ( void ) );
void	obj_update	args( ( void ) );
void	decay_corpse	args( ( OBJ_DATA *corpse ) );



/*
 * Regeneration stuff.
 */
int hit_gain( CHAR_DATA *ch )
{
    int gain = 0;
    int chance = 0;

    if ( IS_NPC(ch) )
    {
	return 0;
    }
    else
    {
	chance = get_curr_tou(ch);

	switch ( ch->position )
	{
	case POS_SLEEPING: chance *= 150; chance /= 100;	break;
	case POS_RESTING:  chance *= 120; chance /= 100;	break;
	}

	if ( ch->pcdata->condition[COND_FULL]   == 0 )
	    chance /= 2;

	if ( ch->pcdata->condition[COND_THIRST] == 0 )
	    chance /= 2;

    }

    if ( IS_AFFECTED(ch, AFF_POISON) )
	chance /= 4;

    chance /= 5;

    while ( chance > 0 )
    {
	if ( number_percent() < chance )
	    gain++;
	chance -= 100;
    }

    return UMIN(gain, ch->max_hit - ch->hit[HIT_LOC_BODY]);
}



int mana_gain( CHAR_DATA *ch )
{
    int gain = 0;
    int chance = 0;

    if ( IS_NPC(ch) )
    {
	return 0;
    }
    else
    {
	chance = get_curr_int( ch ); 

	switch ( ch->position )
	{
	case POS_SLEEPING: chance = 0;	break;
	case POS_RESTING:  chance *= 150; chance /= 100;	break;
	}

	if ( ch->pcdata->condition[COND_FULL]   == 0 )
	    chance /= 2;

	if ( ch->pcdata->condition[COND_THIRST] == 0 )
	    chance /= 2;

    }

    if ( IS_AFFECTED( ch, AFF_POISON ) )
	chance /= 4;

    chance /= 5;

    while ( chance > 0 )
    {
	if ( number_percent() < chance )
	    gain++;
	chance -= 100;
    }

    return UMIN(gain, ch->max_mana - ch->mana);
}



int move_gain( CHAR_DATA *ch )
{
    int gain = 0;
    int chance = 0;

    if ( IS_NPC(ch) )
    {
	return 0;
    }
    else
    {
	chance = get_curr_dex(ch);

	switch ( ch->position )
	{
	case POS_SLEEPING: chance *= 4;		break;
	case POS_RESTING:  chance *= 3;	break;
	case POS_STANDING: chance = 0; break;
	}

	if ( ch->pcdata->condition[COND_FULL]   == 0 )
	    chance /= 2;

	if ( ch->pcdata->condition[COND_THIRST] == 0 )
	    chance /= 2;
    }

    if ( IS_AFFECTED(ch, AFF_POISON) )
	chance /= 4;

    chance /= 5;

    while ( chance > 0 )
    {
	if ( number_percent() < chance )
	    gain++;
	chance -= 100;
    }

    return UMIN(gain, ch->max_move - ch->move);
}



void gain_condition( CHAR_DATA *ch, int iCond, int value )
{
    int condition;

    if ( value == 0 || IS_NPC(ch) || ch->level >= LEVEL_HERO )
	return;

    condition				= ch->pcdata->condition[iCond];

    if (condition < 0)
	return;

    ch->pcdata->condition[iCond]	= UMAX( 0, condition + value );

    if ( ch->pcdata->condition[iCond] == 0 )
    {
	switch ( iCond )
	{
	case COND_FULL:
	    send_to_char( "You are hungry.\n\r",  ch );
	    break;

	case COND_THIRST:
	    send_to_char( "You are thirsty.\n\r", ch );
	    break;

	case COND_DRUNK:
	    if ( condition != 0 )
		send_to_char( "You are sober.\n\r", ch );
	    break;
	}
    }

    return;
}



/*
 * Mob autonomous action.
 * This function takes 25% to 35% of ALL Merc cpu time.
 * -- Furey
 */
void mobile_update( void )
{
    CHAR_DATA *ch;
    CHAR_DATA *ch_next;
    CHAR_DATA *vch;
    ROOM_INDEX_DATA *in_room;
    AFFECT_DATA *paf;
    AFFECT_DATA *paf_next;
    EXIT_DATA *pexit;
    HUNTING_DATA *h;
    int door;

    /* Examine all mobs. */
    for ( ch = char_list; ch != NULL; ch = ch_next )
    {
	ch_next = ch->next;

	for ( paf = ch->affected; paf != NULL; paf = paf_next )
	{
	    paf_next	= paf->next;
	    if ( paf->duration > 0 )
		paf->duration--;
	    else if ( paf->duration < 0 )
		;
	    else
	    {
		if ( paf_next == NULL
		||   paf_next->type != paf->type
		||   paf_next->duration > 0 )
		{
		    if ( paf->type > 0 && skill_table[paf->type].msg_off )
		    {
			send_to_char( skill_table[paf->type].msg_off, ch );
			send_to_char( "\n\r", ch );
		    }
		}
	  
		affect_remove( ch, paf );
	    }
	}

	/* examine delayed functions */
	if ( ch->predelay_time > 0 )
	{
	    ch->predelay_time--;

	    if ( ch->predelay_time == 0 )
	    {
		if ( ch->predelay_info != NULL )
		{
		    (*ch->predelay_info->fnptr) ( ch, ch->predelay_info->argument );

		    free_predelay( ch->predelay_info );
		    ch->predelay_info = NULL;

		    continue;
		}
	    }
	}

	if ( !IS_NPC(ch) || ch->in_room == NULL || IS_AFFECTED(ch, AFF_CHARM) )
	    continue;

	if ( ch->wait > 0 )
	{
	    ch->wait--;
	    continue;
	}

	if ( number_bits( 9 ) > ch->speed )
	    continue;

	if ( ch->position != POS_STANDING )
	{
	    do_stand( ch, "" );
	    continue;
	}

	/* Examine call for special procedure */
	if ( ch->spec_fun != 0 )
	{
	    if ( (*ch->spec_fun) ( NULL, NULL, NULL, NULL, ch ) )
		continue;
	}

	/* That's all for sleeping / busy monster */
	if ( ch->position != POS_STANDING || ch->fighting != NULL )
	    continue;

	in_room = ch->in_room;


/**** grudges ****/
	/* memory */
	for ( h = ch->hunting; h != NULL; h = h->next )
	{
	    if ( h->victim
	    && h->victim->in_room == in_room
	    && can_see( ch, h->victim ) )
	    {
		one_hit( ch, h->victim, TYPE_UNDEFINED );
		return;
	    }
	}

	/* memory to next room */
       if ( !IS_SET(ch->act, ACT_SENTINEL) )
	for ( h = ch->hunting; h != NULL; h = h->next )
	{
	    if ( h->victim == NULL )
	      continue;

	    for ( door = 0; door < 6; door++ )
	    {
		if ( ( pexit = in_room->exit[door] ) != NULL )
		{
		    if ( h->victim->in_room == pexit->to_room )
		    {
			move_char( ch, door );
			continue;
		    }
		}
	    }
	}
/**** end grudges ****/


/****  aggression  ****/
       if ( ch->aggression != 0 )
       {
	int agg = 0;
	int a;
	CHAR_DATA *ach = NULL;

	for ( vch = in_room->people; vch != NULL; vch = vch->next_in_room )
	{
	    if ( vch == ch )
		continue;

	    a = get_aggr_factor( ch, vch );

	    if ( a > agg )
	    {
		agg = a;
		ach = vch;
	    }
	}

	if ( agg != 0 )
	{
	    one_hit( ch, ach, TYPE_UNDEFINED );
	    continue;
	}
       }

	if ( IS_SET( ch->aggression, AGGRESSIVE_LONG_DISTANCE ) )
	{
	    CHAR_DATA *vch = NULL;

	    door = dice(1,6)-1;

	    if ( (pexit = in_room->exit[door]) != NULL && pexit->to_room != NULL)
	    {
		for ( vch = pexit->to_room->people; vch != NULL; vch = vch->next )
		{
		    if ( get_aggr_factor( ch, vch ) )
		    {
			move_char( ch, door );
			continue;
		    }
		}
	    }
	}
/****  end aggression  ****/

	/* Scavenge */
	if ( IS_SET(ch->act, ACT_SCAVENGER)
	&&   ch->in_room->contents != NULL
	&&   number_bits( 5 ) == 0 )
	{
	    OBJ_DATA *obj;
	    OBJ_DATA *obj_best = NULL;
	    int max = 0;

	    for ( obj = ch->in_room->contents; obj; obj = obj->next_content )
	    {
		if ( CAN_WEAR(obj, ITEM_TAKE) && obj->cost > max )
		{
		    obj_best    = obj;
		    max         = obj->cost;
		}
	    }

	    if ( obj_best )
	    {
		obj_from_room( obj_best );
		obj_to_char( obj_best, ch );
		act( "$n gets $p.", ch, NULL, obj_best, NULL, NULL, TO_ROOM, SENSE_SIGHT );
		WAIT_STATE( ch, 20 );
		continue;
	    }
	}

	/* Wander */
	if ( !IS_SET(ch->act, ACT_SENTINEL)
/* dont need, is checked above */
/*
	&&   ch->fighting == NULL
*/
/* raised from 7 to 8 after lowering pulse to .25 */
	&& ( door = number_bits( 8 ) ) <= 5
	&& ( pexit = ch->in_room->exit[door] ) != NULL
	&&   pexit->to_room != NULL
	&&   !IS_SET(pexit->exit_flags, EX_CLOSED)
	&&   !IS_SET(pexit->to_room->room_flags, ROOM_NO_MOB)
	&& ( !IS_SET(ch->act, ACT_STAY_AREA)
	||   pexit->to_room->area == ch->in_room->area ) )
	{
	    move_char( ch, door );
	    WAIT_STATE( ch, 16 );
	}

    }

    return;
}



/*
 * Update the weather.
 */
void weather_update( void )
{
    char buf[MAX_STRING_LENGTH];
    DESCRIPTOR_DATA *d;
    int diff;

    buf[0] = '\0';

    switch ( ++time_info.hour )
    {
    case  5:
	weather_info.sunlight = SUN_LIGHT;
	strcat( buf, "The day has begun.\n\r" );
	break;

    case  6:
	weather_info.sunlight = SUN_RISE;
	strcat( buf, "The sun rises in the east.\n\r" );
	break;

    case 19:
	weather_info.sunlight = SUN_SET;
	strcat( buf, "The sun slowly disappears in the west.\n\r" );
	break;

    case 20:
	weather_info.sunlight = SUN_DARK;
	strcat( buf, "The night has begun.\n\r" );
	break;

    case 24:
	time_info.hour = 0;
	time_info.day++;
	break;
    }

    if ( time_info.day   >= 35 )
    {
	time_info.day = 0;
	time_info.month++;
    }

    if ( time_info.month >= 17 )
    {
	time_info.month = 0;
	time_info.year++;
    }

    /*
     * Weather change.
     */
    if ( time_info.month >= 9 && time_info.month <= 16 )
	diff = weather_info.mmhg >  985 ? -2 : 2;
    else
	diff = weather_info.mmhg > 1015 ? -2 : 2;

    weather_info.change   += diff * dice(1, 4) + dice(2, 6) - dice(2, 6);
    weather_info.change    = UMAX(weather_info.change, -12);
    weather_info.change    = UMIN(weather_info.change,  12);

    weather_info.mmhg += weather_info.change;
    weather_info.mmhg  = UMAX(weather_info.mmhg,  960);
    weather_info.mmhg  = UMIN(weather_info.mmhg, 1040);

    switch ( weather_info.sky )
    {
    default: 
	bug( "Weather_update: bad sky %d.", weather_info.sky );
	weather_info.sky = SKY_CLOUDLESS;
	break;

    case SKY_CLOUDLESS:
	if ( weather_info.mmhg <  990
	|| ( weather_info.mmhg < 1010 && number_bits( 2 ) == 0 ) )
	{
	    strcat( buf, "The sky is getting cloudy.\n\r" );
	    weather_info.sky = SKY_CLOUDY;
	}
	break;

    case SKY_CLOUDY:
	if ( weather_info.mmhg <  970
	|| ( weather_info.mmhg <  990 && number_bits( 2 ) == 0 ) )
	{
	    strcat( buf, "It starts to rain.\n\r" );
	    weather_info.sky = SKY_RAINING;
	}

	if ( weather_info.mmhg > 1030 && number_bits( 2 ) == 0 )
	{
	    strcat( buf, "The clouds disappear.\n\r" );
	    weather_info.sky = SKY_CLOUDLESS;
	}
	break;

    case SKY_RAINING:
	if ( weather_info.mmhg <  970 && number_bits( 2 ) == 0 )
	{
	    strcat( buf, "Lightning flashes in the sky.\n\r" );
	    weather_info.sky = SKY_LIGHTNING;
	}

	if ( weather_info.mmhg > 1030
	|| ( weather_info.mmhg > 1010 && number_bits( 2 ) == 0 ) )
	{
	    strcat( buf, "The rain stopped.\n\r" );
	    weather_info.sky = SKY_CLOUDY;
	}
	break;

    case SKY_LIGHTNING:
	if ( weather_info.mmhg > 1010
	|| ( weather_info.mmhg >  990 && number_bits( 2 ) == 0 ) )
	{
	    strcat( buf, "The lightning has stopped.\n\r" );
	    weather_info.sky = SKY_RAINING;
	    break;
	}
	break;
    }

    if ( buf[0] != '\0' )
    {
	for ( d = descriptor_list; d != NULL; d = d->next )
	{
	    if ( d->connected == CON_PLAYING
	    &&   VIEW_OUTSIDE(d->character)
	    &&   IS_AWAKE(d->character) )
		send_to_char( buf, d->character );
	}
    }

    return;
}



/*
 * Update all chars, including mobs.
 * This function is performance sensitive.
 */
void char_update( void )
{   
    CHAR_DATA *ch;
    CHAR_DATA *ch_next;
    CHAR_DATA *ch_save;
    CHAR_DATA *ch_quit;
    time_t save_time;

    save_time	= current_time;
    ch_save	= NULL;
    ch_quit	= NULL;
    for ( ch = char_list; ch != NULL; ch = ch_next )
    {
	sh_int light, vision;

	ch_next = ch->next;

	/*
	 * Find dude with oldest save time.
	 */
	if ( !IS_NPC(ch)
	&& ( ch->desc == NULL || ch->desc->connected == CON_PLAYING )
	&&   ch->save_time < save_time )
	{
	    ch_save	= ch;
	    save_time	= ch->save_time;
	}

	if ( ch->position >= POS_STUNNED && IS_NPC( ch ) )
	{
	    if ( ch->hit[HIT_LOC_BODY]  < ch->max_hit )
		ch->hit[HIT_LOC_BODY] = UMIN( ch->max_hit, ch->hit[HIT_LOC_BODY] + get_curr_tou( ch )/5 );

	    if ( ch->mana < ch->max_mana )
		ch->mana = UMIN( ch->max_mana, ch->mana + get_curr_int( ch )/3);

	    if ( ch->move < ch->max_move )
		ch->move = UMIN( ch->max_move, ch->move + get_curr_dex( ch )/5);
	}

	if ( ch->position == POS_STUNNED )
	    update_pos( ch );

	if ( !IS_NPC(ch) && ch->level < LEVEL_IMMORTAL )
	{
	    OBJ_DATA *obj;

	    if ( ( obj = get_eq_char( ch, gn_wear_secondary ) ) != NULL
	    &&   obj->item_type == ITEM_LIGHT
	    &&   obj->value[2] >= 0
	    &&   obj->value[0] != 0 )
	    {
		if ( obj->value[2] == 0 && ch->in_room != NULL )
		{
		    ch->in_room->light_mod -= (obj->value[0]/2);
		    ch->light -= (obj->value[0]/2);
		    obj->value[0] -= (obj->value[0]/2);
		    if ( obj->value[0] <= 1 && obj->value[0] >= -1 )
		    {
			ch->in_room->light_mod -= obj->value[0];
			ch->light -= obj->value[0];
			obj->value[0] = 0;
			act( "$p goes out.", ch, NULL, obj, NULL, NULL, TO_ROOM, SENSE_SIGHT );
			act( "$p goes out.", ch, NULL, obj, NULL, NULL, TO_CHAR, SENSE_SIXTH );
		    }
		    else
		    {
			act( "$p fades slowly.", ch, NULL, obj, NULL, NULL, TO_ROOM, SENSE_SIGHT );
			act( "$p fades slowly.", ch, NULL, obj, NULL, NULL, TO_CHAR, SENSE_SIXTH );
		    }
		}
		else
		    obj->value[2]--;
	    }
	    ch->timer++;
/* no void for now */
/*
	    if ( ++ch->timer >= 12 )
	    {
		if ( ch->was_in_room == NULL && ch->in_room != NULL )
		{
		    ch->was_in_room = ch->in_room;
		    if ( ch->fighting != NULL )
			stop_fighting( ch, TRUE );
		    act( "$n disappears into the void.",
			ch, NULL, NULL, NULL, NULL, TO_ROOM, SENSE_SIGHT );
		    send_to_char( "You disappear into the void.\n\r", ch );
		    save_char_obj( ch );
		    char_from_room( ch );
		    char_to_room( ch, get_room_index( ROOM_VNUM_LIMBO ), FALSE );
		}
	    }
*/

/* no auto-quit for now */
/*
	    if ( ch->timer > 30 )
		ch_quit = ch;
*/

	    gain_condition( ch, COND_DRUNK,  -1 );
	    gain_condition( ch, COND_FULL,   -1 );
	    gain_condition( ch, COND_THIRST, -1 );
	}

	/* update skill delays */
	if ( !IS_NPC( ch ) )
	{
	    SKILL_DELAY_DATA *delay;
	    SKILL_DELAY_DATA *delay_next;

	    for ( delay = ch->pcdata->skill_delays; delay != NULL; delay = delay_next )
	    {
		delay_next = delay->next;

		if ( --delay->duration <= 0 )
		{
		    delay_remove( ch, delay );
		}
	    }
	}

	/* adjust vision */
      if ( IS_NPC(ch) )
      {
	int mod;

	vision = ch->vision_current;
	light = room_light( ch->in_room );
	if ( vision != light
	&& ((vision > ch->vision_min && vision < ch->vision_max )
	|| ( ch->vision_min == ch->vision_max ) ) )
	{
	    mod = light - vision;

	    if ( vision < ch->vision_min )
		mod = URANGE( 0, mod, 50 );
	    else
		mod = URANGE( -50, mod, 0 );

	    ch->vision_current = vision + mod;
	}
      }

	/*
	 * Careful with the damages here,
	 *   MUST NOT refer to ch after damage taken,
	 *   as it may be lethal damage (on NPC).
	 */
	if ( IS_AFFECTED(ch, AFF_POISON) )
	{
	    damage( ch, ch, 2, gsn_poison, NULL, gn_wear_none );
	}
	else if ( ch->position == POS_INCAP )
	{
	    damage( ch, ch, 1, gsn_bleeding, NULL, gn_wear_none );
	}
	else if ( ch->position == POS_MORTAL )
	{
	    damage( ch, ch, 2, gsn_bleeding, NULL, gn_wear_none );
	}
    }

    /*
     * Autosave and autoquit.
     * Check that these chars still exist.
     */
    if ( ch_save != NULL || ch_quit != NULL )
    {
	for ( ch = char_list; ch != NULL; ch = ch_next )
	{
	    ch_next = ch->next;
	    if ( ch == ch_save )
		save_char_obj( ch );
	    if ( ch == ch_quit )
		do_quit( ch, "" );
	}
    }

    return;
}


/*
 * Update all player characters.
 */
void pc_update( void )
{
    DESCRIPTOR_DATA *d;
    CHAR_DATA *ch;
    int vision;

    for ( d = descriptor_list; d != NULL; d = d->next )
    {
	if ( d->connected != CON_PLAYING )
	    continue;

	if ( ( ch = d->character ) == NULL )
	    continue;

	if ( IS_NPC( ch ) )
	    continue;

	if ( ch->position >= POS_STUNNED && ch->fighting == NULL )
	{
	    if ( ch->hit[HIT_LOC_BODY]  < ch->max_hit )
		ch->hit[HIT_LOC_BODY]  += hit_gain(ch);

	    if ( ch->mana < ch->max_mana )
		ch->mana += mana_gain(ch);

	    if ( ch->move < ch->max_move )
		ch->move += move_gain(ch);
	}

	if ( ch->position == POS_STUNNED )
	    update_pos( ch );

	vision = ch->vision_current;
	if ( room_light( ch->in_room ) > vision && vision < ch->vision_max )
	    ch->vision_current+=2;

	if ( room_light( ch->in_room ) < vision && vision > ch->vision_min )
	    ch->vision_current-=2;
    }

    return;
}


/*
 * Update all objs.
 * This function is performance sensitive.
 */
void obj_update( void )
{   
    OBJ_DATA *obj;
    OBJ_DATA *obj_next;

    for ( obj = object_list; obj != NULL; obj = obj_next )
    {
	CHAR_DATA *rch;
	char *message;

	obj_next = obj->next;

	if ( obj->timer <= 0 || --obj->timer > 0 )
	    continue;

	switch ( obj->item_type )
	{
	default:              message = "$p vanishes.";         break;
	case ITEM_FOUNTAIN:   message = "$p dries up.";         break;
	case ITEM_CORPSE:     message = "$p is eaten by a quivering hoard of maggots."; break;
	case ITEM_FOOD:       message = "$p decomposes.";	break;
	}

	if ( obj->carried_by != NULL )
	{
	    act( message, obj->carried_by, NULL, obj, NULL, NULL, TO_CHAR, SENSE_SIXTH );
	}
	else if ( obj->in_room != NULL
	&&      ( rch = obj->in_room->people ) != NULL )
	{
	    act( message, rch, NULL, obj, NULL, NULL, TO_ROOM, SENSE_SIGHT );
	    act( message, rch, NULL, obj, NULL, NULL, TO_CHAR, SENSE_SIGHT );
	}

	if ( obj->item_type == ITEM_CORPSE )
	    decay_corpse( obj );
	else
	    extract_obj( obj, TRUE );
    }

    return;
}


void decay_corpse( OBJ_DATA *corpse )
{
    OBJ_DATA *obj;
    OBJ_DATA *obj_next;
    ROOM_INDEX_DATA *in_room = NULL;
    bool fAll = FALSE;

    if ( corpse->carried_by == NULL && corpse->in_room == NULL )
    {
	fAll = TRUE;
        extract_obj( corpse, TRUE );
	return;
    }
    else if ( corpse->carried_by != NULL )
    {
	if ( corpse->carried_by->in_room == NULL )
	    fAll = TRUE;
	else
	    in_room = corpse->carried_by->in_room;
    }
    else
    {
	in_room = corpse->in_room;
    }

    for ( obj = corpse->contains ; obj != NULL ; obj = obj_next )
    {
	obj_next = obj->next_content;

	obj_from_obj( obj );
	obj_to_room( obj, in_room, TRUE );
    }

    extract_obj( corpse, TRUE );

    return;
}



/*
 * Handle all kinds of updates.
 * Called once per pulse from game loop.
 * Random times to defeat tick-timing clients and players.
 */
void update_handler( void )
{
    static  int     pulse_area;
    static  int     pulse_mobile;
    static  int     pulse_violence;
    static  int     pulse_weather;
    static  int     pulse_point;
    static  int     pulse_pc_update;
    static  int     pulse_environment;

    if ( --pulse_area     <= 0 )
    {
	pulse_area	= PULSE_AREA;
	area_update	( );
    }

    if ( --pulse_mobile   <= 0 )
    {
	pulse_mobile	= PULSE_MOBILE;
	mobile_update	( );
    }

    if ( --pulse_violence <= 0 )
    {
	pulse_violence	= PULSE_VIOLENCE;
	violence_update	( );
    }

    if ( --pulse_weather <= 0 )
    {
	pulse_weather = PULSE_WEATHER;
	weather_update ( );
	obj_update	( );
    }

    if ( --pulse_environment <= 0 )
    {
      pulse_environment = PULSE_ENVIRONMENT;
      environment_interval++;
      write_environment( FALSE );
    }

    if ( --pulse_point    <= 0 )
    {
	pulse_point     = PULSE_TICK;
	char_update	( );
    }

    if ( --pulse_pc_update <= 0 )
    {
	pulse_pc_update = PULSE_PC_UPDATE;
	pc_update ( );
    }

    tail_chain( );
    return;
}
